==> eagle-i-ui-gwt-1.0-MS6.03-sources.jar: org/eaglei/ui/gwt/search/results/OntologyDropdown.java <==
package org.eaglei.ui.gwt.search.results;

import java.util.List;

import org.eaglei.model.EIClass;
import org.eaglei.model.gwt.rpc.ClientModelManager;
import org.eaglei.model.gwt.rpc.ClientModelManager.SubClassCallback;
import org.eaglei.suggest.client.OntologyDropdownBase;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.DeferredCommand;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.TreeItem;

public class OntologyDropdown extends OntologyDropdownBase {
    

    public OntologyDropdown() {
    	super.init();
    }
    
    /**
     * Popup display click event.
     */
    protected void doOnClick(ClickEvent event) {
        if (rootClass != null && rootListLoading) {
            DeferredCommand.addCommand(new Command() {

                public void execute() {
                    if (! rootListLoading) {
                        return;
                    }
                    
                    ClientModelManager.INSTANCE.getSubClasses(rootClass, new SubClassCallback() {

                        public void onSuccess(EIClass result) {
                        	// check that another thread hasn't already popuplated
                            // the list or there has been a switch to another resource type.
//                            if (! rootListLoading || ! result.equals(rootClass)) {
//                                return;
//                            }
                            tree.clear();
                           
                            List<EIClass> subclasses = result.getSubClasses();
                            for (EIClass resource : subclasses) {
                                TreeItem item;
                                ResourceTreeWidget w = new ResourceTreeWidget(resource);
                                item = tree.addItem(w);
                                if (resource.hasSubClass()) {
                                	populateChildList(item);
                              //      item.addItem(ApplicationImages.LOADING);
                                    w.childListLoading = true;
                                }
                            }
                            rootListLoading = false;
                            setPopupHeight();
                        }

                        
                    });
                }

            });
        }
        if (popup != null) {
            popup.setVisible(false);
            popup.show();
            setPopupHeight();
        }
    }

    @Override
    protected void populateChildList(final TreeItem item) { 	
        final ResourceTreeWidget w = (ResourceTreeWidget) item.getWidget();
        EIClass resource = w.getResource();
        if (resource.getSubClasses() != null) {
            populateChildList(item, w, resource.getSubClasses());
        } else {
            ClientModelManager.INSTANCE.getSubClasses(resource, new SubClassCallback() {

				@Override
				public void onSuccess(EIClass result) {
					 // Check that the child list hasn't already been
                    // populated by another thread.
                    if (! w.childListLoading) {
                        return;
                    }
                    populateChildList(item, w, result.getSubClasses());
				}
            });
        }
    }
    
    @Override
    protected void populateChildList(final TreeItem parent, ResourceTreeWidget parentWidget, List<EIClass> subclasses) {
    
        parent.removeItems();
        for (EIClass r : subclasses) {
            TreeItem child;
            ResourceTreeWidget childWidget = new ResourceTreeWidget(r);
            child = parent.addItem(childWidget);
            if (r.hasSubClass()) {
             //   child.addItem(ApplicationImages.LOADING);
                childWidget.childListLoading = true;
            }
        }
        parentWidget.childListLoading = false;
        setPopupHeight();
    }
    
    private void setPopupHeight() {
        DeferredCommand.addCommand(new Command() {
            public void execute() {
                int h = tree.getOffsetHeight();
                int maxHeight = computeMaxHeight();
                if (h > maxHeight) {
                    scroll.setHeight(maxHeight + "px");
                } else {
                    scroll.getElement().getStyle().clearHeight();
                }
                popup.showRelativeTo(p);
            }
        });
    }
    
    private int computeMaxHeight() {
        // Calculate top position for the popup
        int top = p.getAbsoluteTop();

        // Make sure scrolling is taken into account, since
        // widget.getAbsoluteTop() takes scrolling into account.
        int windowTop = Window.getScrollTop();
        int windowBottom = Window.getScrollTop() + Window.getClientHeight();

        // Distance from the top edge of the window to the top edge of the
        // widget
        int distanceFromWindowTop = top - windowTop;

        // Distance from the bottom edge of the window to the bottom edge of
        // the widget
        int distanceToWindowBottom = windowBottom
            - (top + p.getOffsetHeight());

        // Prefer displaying below
        if (distanceToWindowBottom > 200) {
            return distanceToWindowBottom - 10;
        } else if (distanceFromWindowTop > 200) {
            return distanceFromWindowTop - 10;
        } else {
            return distanceToWindowBottom > distanceFromWindowTop ? distanceToWindowBottom - 10 : distanceFromWindowTop - 10;
        }
    }
}

==> eagle-i-ui-gwt-1.2-MS1.00-sources.jar: org/eaglei/ui/gwt/search/results/OntologyDropdown.java <==
package org.eaglei.ui.gwt.search.results;

import java.util.List;

import org.eaglei.model.EIClass;
import org.eaglei.model.gwt.rpc.ClientModelManager;
import org.eaglei.model.gwt.rpc.ClientModelManager.SubClassCallback;
import org.eaglei.suggest.client.OntologyDropdownBase;
import org.eaglei.ui.gwt.ApplicationResources;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.DeferredCommand;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.TreeItem;

public class OntologyDropdown extends OntologyDropdownBase {
    
    private static final Image LOADING = new Image(ApplicationResources.INSTANCE.loadingSmall());
 
    public OntologyDropdown() {
    	super.init();
    }
    
    public void setResource(EIClass resource) {
        rootClass = resource;
        if (tree != null) {
            tree.clear();
            scroll.getElement().getStyle().clearHeight();
        }
        if (rootClass.hasSubClass()) {
            if (tree != null) {
                 tree.add(LOADING);
            }
            rootListLoading = true;
        } else {
            // Probably should assert...
            if (tree != null) {
                tree.addItem("No types are available");
            }
        }
    }

    /**
     * Popup display click event.
     */
    protected void doOnClick(ClickEvent event) { 
        if (rootClass != null && rootListLoading) {
            DeferredCommand.addCommand(new Command() {

                public void execute() {
                    if (! rootListLoading) {
                        return;
                    }
                    
                    ClientModelManager.INSTANCE.getSubClasses(rootClass, new SubClassCallback() {

						@Override
						public void onSuccess(EIClass c) {
						    List<EIClass> result = c.getSubClasses();
                            tree.clear();
                            for (EIClass resource : result) {
                                TreeItem item;
                                ResourceTreeWidget w = new ResourceTreeWidget(resource);
                                item = tree.addItem(w);
                                if (resource.hasSubClass()) {
                                    item.addItem(LOADING);
                                    w.childListLoading = true;
                                }
                            }
                            rootListLoading = false;
                            setPopupHeight();
							
						}
                        
                    });
                }

            });
        }
        if (popup != null) {
            popup.setVisible(false);
            popup.show();
            setPopupHeight();
        }
    }

    @Override
    protected void populateChildList(final TreeItem item) { 
        final ResourceTreeWidget w = (ResourceTreeWidget) item.getWidget();
        EIClass resource = w.getResource(); 
        if (resource.getSubClasses() != null) {
            populateChildList(item, w, resource.getSubClasses());
        } else {
            ClientModelManager.INSTANCE.getSubClasses(resource, new SubClassCallback() {

				@Override
				public void onSuccess(EIClass c) {
                    List<EIClass> result = c.getSubClasses();
					  // Check that the child list hasn't already been 
                    // populated by another thread.
                    if (! w.childListLoading) {
                        return;
                    }
                    populateChildList(item, w, result);
					
				}
                
            });
        }
    }
    
    @Override
    protected void populateChildList(final TreeItem parent, ResourceTreeWidget parentWidget, List<EIClass> subclasses) {
        parent.removeItems();
        for (EIClass r : subclasses) {
            TreeItem child;
            ResourceTreeWidget childWidget = new ResourceTreeWidget(r);
            child = parent.addItem(childWidget);
            if (r.hasSubClass()) {
                child.addItem(LOADING);
                childWidget.childListLoading = true;
            }
        }
        parentWidget.childListLoading = false;
        setPopupHeight();
    }
    
    private void setPopupHeight() {
        DeferredCommand.addCommand(new Command() {
            public void execute() {
                int h = tree.getOffsetHeight();
                int maxHeight = computeMaxHeight();
                if (h > maxHeight) {
                    scroll.setHeight(maxHeight + "px");
                } else {
                    scroll.getElement().getStyle().clearHeight();
                }
                popup.showRelativeTo(p);
            }
        });
    }
    
    private int computeMaxHeight() {
        // Calculate top position for the popup
        int top = p.getAbsoluteTop();

        // Make sure scrolling is taken into account, since
        // widget.getAbsoluteTop() takes scrolling into account.
        int windowTop = Window.getScrollTop();
        int windowBottom = Window.getScrollTop() + Window.getClientHeight();

        // Distance from the top edge of the window to the top edge of the
        // widget
        int distanceFromWindowTop = top - windowTop;

        // Distance from the bottom edge of the window to the bottom edge of
        // the widget
        int distanceToWindowBottom = windowBottom
            - (top + p.getOffsetHeight());

        // Prefer displaying below
        if (distanceToWindowBottom > 200) {
            return distanceToWindowBottom - 10;
        } else if (distanceFromWindowTop > 200) {
            return distanceFromWindowTop - 10;
        } else {
            return distanceToWindowBottom > distanceFromWindowTop ? distanceToWindowBottom - 10 : distanceFromWindowTop - 10;
        }
    }
}
